from matplotlib import pyplot as plt
import scipy.io
import numpy
import math
import Image
from matplotlib.pyplot import *
from matplotlib.pylab import *
import os
from matplotlib import pyplot
import matplotlib.gridspec as gridspec
#######################################################################################################



def loadReceptors(path):
    "loads list of receptors from a file"
    X = []
    Y = []
    names = []
    f = open(path, "r")
    s = f.readlines()
    InputTuple = []
    for line in s[0:len(s)]:
        singleLine = line.rstrip()
        InputTuple.append(singleLine)
        name = singleLine[14:42]
        north = float(singleLine[42:54])
        east = float(singleLine[54:62])
        Y.append(float(east))
        X.append(float(north))
        names.append(name)
    f.close()
    return [X, Y, names]



#vyrizne z mapy na ceste path a puvodnim rozmeru a1 novou mapu s rozmerem a2
def get_square_map_of_given_side(path, a1, a2):
    #mam tu jen x0, pac mapa je ctvercova....
    img = Image.open(path)
    x0 = img.size[0] #sirka obrazku
    y0 = img.size[1] #vyska obrazku
    x1 = int(x0/2 - x0*a2/(2*a1))
    y1 = int(y0/2 + y0*a2/(2*a1))
    cropped = img.crop([ x1, x1, y1, y1])
    #plt.imshow(cropped, extent=(-a2/2, a2/2, -a2/2, a2/2), origin='lower')
    #plt.show()
    return cropped


def main(out_path, pics_path, filename):
    QsP = scipy.io.loadmat(out_path+os.sep+filename)["Qs_after_resampling"] 

    d = scipy.io.loadmat("%s_grid_18" % (out_path+os.sep+filename))["this_step_DOSE"]
    
    window = 6
    data = []
    
    print "Processing files %s, %s." %  (out_path+os.sep+filename, "%s_grid_18" % (out_path+os.sep+filename))

    for t in range(QsP.shape[2]):#1-30
        sub = []#range(t+1) #

        for i in range(t+1):
            sub.append(numpy.log(QsP[t,:,i]))
        data.append(sub)

    

        
    dimX = 41
    dimY = 41
    dS = 1.0
    deltaStep = dS
    
    fig = plt.figure(figsize=(16,8.5))
    
    zero_rel = 1.0 #e10
    true = numpy.array( [zero_rel, zero_rel, zero_rel, zero_rel, zero_rel, zero_rel, 1.0E+16, 5.0E+16, 4.0E+16, 3.0E+16, 2.0E+16, 1.0E+16, zero_rel, zero_rel, zero_rel, zero_rel, zero_rel, zero_rel])
    #limit = numpy.ones(12.0)

    #text sizes
    ts = 14
    ts2 = 13

  
    index = 0
    zaj =  [5,11,17]
    
    #customizing subplots
    gs = gridspec.GridSpec(3, 3,height_ratios=[1,1,2])
    axy = []
    for tt in range(len(zaj)):
        
        t = zaj[tt]
        index +=1
        ax = fig.add_subplot(gs[index-1])

        ax.plot(range(1,7),numpy.log(true[t-6+1:t+1]), 'k--', label='true release')
        curr_data =  data[t][t-window+1:t+1]
        ax.boxplot(curr_data)        
 
 
        
        ax.set_title(r"$p(Q_{t-5:t}|y_{1:t})$, $t=%d$" % (t+1), size=ts)
        ax.spines['bottom'].set_visible(False)
        ax.get_xaxis().tick_top()
 
        ylabels = [1.0E16, 5.0E16]#, 1.0E17]
        ylims = [7.0E15, 7.0E16]

        ylabels_str = []
        for lab in ylabels:
            ylabels_str.append("%2.1E" % lab)
        ax.set_yticks(numpy.log(numpy.array(ylabels)))
        ax.set_yticklabels(ylabels_str)
        ax.set_ylim(math.log(ylims[0]), math.log(ylims[len(ylabels)-1]))
        
            
        if (t != 5):
            ax.get_yaxis().set_visible(False)      
        ax.get_xaxis().set_visible(False) 
                 
        axy.append(ax)
      

        
    for tt in range(len(zaj)):
        t = zaj[tt]
        index +=1
        
        ax = fig.add_subplot(gs[index-1])
        shift = 0.019
        
        x0 = ax.get_position()._get_x0()
        y0 = ax.get_position()._get_y0()
        w = ax.get_position()._get_width()
        h = ax.get_position()._get_height()
        ax.set_position([x0, y0+shift, w, h])        
        
        ax.plot(range(1,7),numpy.log(true[t-6+1:t+1]), 'k--', label='true release')
        curr_data =  data[t][t-window+1:t+1]
        ax.boxplot(curr_data) 
        
        if (t != 5):
            ax.get_yaxis().set_visible(False)

 
        ylabels = [1.0E9, 1.0E10, 1.0E11] 
        ylims = [5.0E8,  5.0E11]
        ylabels_str = []
        for lab in ylabels:
            ylabels_str.append("%2.1E" % lab)
        ax.set_yticks(numpy.log(numpy.array(ylabels)))
        ax.set_yticklabels(ylabels_str)
        ax.set_ylim(math.log(ylims[0]), math.log(ylims[len(ylims)-1]))
        ax.spines['top'].set_visible(False)
        ax.get_xaxis().tick_bottom()
        ax.set_xticklabels(range(t+1-12+1, t+1+1))
        if (tt==1):
            ax.legend(loc="lower right")
        ax.patch.set_alpha(0.0)

    for ax in axy:
        d0 = math.log(7.E15)

        ax.plot((0.25, 0.75 ), (d0*0.998, d0*1.002) , 'k', clip_on=False)
        ax.plot((6.25, 6.75 ), (d0*0.998, d0*1.002) , 'k', clip_on=False)
        d0 = math.log(5.E15)       
        ax.plot((0.25, 0.75  ), (d0*0.998, d0*1.002) , 'k', clip_on=False)
        ax.plot((6.25, 6.75 ), (d0*0.998, d0*1.002) , 'k', clip_on=False)   
    
    receptor_path = ".."+os.sep+"data"+os.sep+"receptory_ETE"
    receptors = []

    #off-grid receptors which we are interested in
    X0, Y0, names = loadReceptors(receptor_path)
    number_of_receptors = len(X0)
    for i in range(len(X0)):
        receptors.append((X0[i], Y0[i], 0.0))
   
    for t in [6,11,17]:
        index +=1        
        ax = fig.add_subplot(gs[index-1])
        x = numpy.arange(-math.floor(dimX/2.0)*1000, math.ceil(dimX/2.0)*1000, deltaStep*1000)
        y = numpy.arange(-math.floor(dimY/2.0)*1000, math.ceil(dimY/2.0)*1000, deltaStep*1000)


        map_path = ".."+os.sep+"data"+os.sep+"1ku275000.png"
        orig_size = 25.0
    
        radius = 22.0
    
        map = get_square_map_of_given_side(map_path, orig_size, radius)


        X, Y = numpy.meshgrid(x, y)

        if (t != 11):
            ax.get_yaxis().set_visible(False)
        
        M = numpy.reshape(d[t,:], (41,41))
           
        logMax = math.log(M.max(), 10.0)
        radMaxima = math.floor(logMax)

        levels = [5.0E-8,    1.0e-06,   1.0e-05, 1.0e-04,   1.0e-03 ,  1.0e-02]

        const = 1.000                 



        ax.set_title(r"Ground-level dose, $t=%d$" % (t+1))
        
        #ax.scatter(X[:], Y[:], marker='p',c='r',s=80,)  #, alpha=0.3)
        ax.scatter(numpy.array(X0[:])/1000., numpy.array(Y0[:])/1000., marker='p',c='r',s=80,)
        #ax.scatter([0.0], [0.0], marker='^',c='b',s=80,)
                
        M1 = numpy.zeros(M.shape)
        for i in range(len(M)):
            for j in range(len(M)):
                M1[i][j] = M[len(M)-1-j][i]
    
        cset1 = ax.contourf(X/1000., Y/1000., M1,  levels, alpha=0.3, cmap=cm.jet, norm=matplotlib.colors.LogNorm(), )
        cset2 = ax.contour(X/1000., Y/1000., M1, levels, colors = 'k', hold='on', linewidths = (1.5,), )
    
        for c in cset2.collections:
            c.set_linestyle('solid')

        ax.imshow(map, extent=(-2.*radius * const-1.3, 2.*radius* const-1.3, -2.*radius* const, 2*radius* const), origin='lower')
        ax.set_xlim(-3.5 * const, 20 * const)
        ax.set_ylim(-3.5 * const, 20 * const)
        ax.set_xlabel("distance [km]", size=ts2)
        if (t==11):
            ax.set_ylabel("distance [km]", size=ts2)
        

    plt.savefig("%s%s_boxplots.pdf" % (pics_path+os.sep, filename))
        
    print "Saved file %s_boxplots.pdf." % (pics_path+os.sep+filename)   

if __name__ == '__main__':
 
    out_path = "output"
    pics_path = "pics"
        
    filename = "results_NR_0"
    #filename = "results_AMIS_0"
    
    main(out_path, pics_path, filename)
