import sys
import scipy
import numpy


class NuclideDB:
    """
    class implementing a simple database with main physical properties of nuclides 
    """
    nuclides     = []
    halfTimes    = []
    physicalType = []

    doseA = []
    doseB = []
    doseMJU = []
    doseE = []
    doseMJUaE = []
    doseCb = []

    def __init__(self):
        "konstruktor"
        self.initializeDB()

    def getIndexOfNuclide(self, nuclide):
        nuclide = nuclide.upper()
        index = -1
        for i in range(len(self.nuclides)):
            if (nuclide == self.nuclides[i]):
                index = i
        if (index < 0 ):
            print "Nuclide %s not found in nuclide DB" % nuclide
            print "Exiting"
            sys.exit(0)

        return index
    
    def getDepoVG(self, nuclide):
        """
        dry deposition rate for appropriate nuclide 
        """
        index = self.getIndexOfNuclide(nuclide)
        type = self.physicalType[index]

        v = 0.0
        
        if (type == "noble"):
            v = 0.0
        elif (type == "aero"):
            v = 0.003
        elif (type == "eliod"):
            v = 0.015

        return v

    def getLambda(self, nuclide):
        """
        decay constants for appropriate nuclides
        """
        index = self.getIndexOfNuclide(nuclide)
        lamb = 0.693/self.halfTimes[index]
        return lamb

    def getDoseA(self, nuclide):
        index = self.getIndexOfNuclide(nuclide)
        return self.doseA[index]

    def getDoseB(self, nuclide):
        index = self.getIndexOfNuclide(nuclide)
        return self.doseB[index]

    def getDoseMJU(self, nuclide):
        index = self.getIndexOfNuclide(nuclide)
        return self.doseMJU[index]


    def getDoseE(self, nuclide):
        index = self.getIndexOfNuclide(nuclide)
        return self.doseE[index]

    def getDoseMJUaE(self, nuclide):
        index = self.getIndexOfNuclide(nuclide)
        return self.doseMJUaE[index]

    def getDoseCb(self, nuclide):
        index = self.getIndexOfNuclide(nuclide)
        return self.doseCb[index]

    def initializeDB(self):                                                                         
        self.nuclides     = ["KR-88", "XE-135", "XE-135M",  "XE-138",  "AR-41",  "CS-137",  "I-131"] #units:
        self.halfTimes    = [  10224,    32400,       900,     847.8,   6560.0,  9.52e+08, 6.95e+05] #s
        self.physicalType = ["noble",  "noble",   "noble",   "noble",  "noble",    "aero",  "eliod"]
        self.doseA        = [    0.0,      0.0,       0.0,       0.0,    1.040,       0.0,      0.0] #
        self.doseB        = [    0.0,      0.0,       0.0,       0.0,   0.0338,       0.0,      0.0] #
        self.doseMJU      = [    0.0,      0.0,       0.0,       0.0,  0.00668,       0.0,      0.0] #m-1
        self.doseE        = [    0.0,      0.0,       0.0,       0.0,  1293000,       0.0,      0.0] #eV
        self.doseMJUaE    = [    0.0,      0.0,       0.0,       0.0, 6.12E-16,       0.0,      0.0] #Gy m2
        self.doseCb       = [    0.0,      0.0,       0.0,       0.0,    0.765,       0.0,      0.0] #Sv/Gy



class DoseDict:
            #nactu doseData
    doseDict = {}

    def __init__(self, pathToDoseData):
        self.doseDict = {}
        self.doseData = scipy.io.loadmat(pathToDoseData)["0"]
        dist = [50, 100, 150, 200, 250, 300, 350, 400, 450, 500, 600, 700, 800, 900, 1000, 1200, 1400, 1600, 1800, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 6000, 7000, 8000, 9000, 10000, 11000, 12000, 13000, 14000, 15000, 16000, 17000, 18000, 19000, 20000, 21000, 22000, 23000, 24000, 25000, 26000, 27000, 28000, 29000, 30000]
        sigXYField = numpy.arange(50, 1300, 20)
        sigZField = numpy.arange(50, 1300, 75)

        for i in range(len(dist)):
            for j in range(len(sigXYField)):
                for k in range(len(sigZField)):
                    key = (dist[i], sigXYField[j], sigZField[k])
                    self.doseDict[key] = self.doseData[i][j][k]

        print "Dose data ready!"