#! /usr/bin/python

import sys
import os
sys.path.append("..")

try: #fast cython version
    from puff_cython.puff_tools import Puff
    from puff_cython import ppe 
    print "Cython version imported"   
except: #slow python version
    from puff_python.puff_tools import Puff
    from puff_python import ppe
    print "Python version imported"

import scipy.io
import math
import numpy
import matplotlib.pyplot as plt
from matplotlib.pyplot import *
import scipy.special as sps
import data.nuclideDB as ndb
import time
import pp
import os

    
def loadReceptors(path):
    "nacteni seznamu meresni ze souboru"
    X = []
    Y = []
    names = []
    f = open(path, "r")
    s = f.readlines()
    InputTuple = []
    for line in s[0:len(s)]:
        singleLine = line.rstrip()
        InputTuple.append(singleLine)
        name = singleLine[14:42]
        north = float(singleLine[42:54])
        east = float(singleLine[54:62])
        Y.append(float(east))
        X.append(float(north))
        names.append(name)
    f.close()
    return [X, Y, names]    


def main(out_path, filename):
    
    dic = scipy.io.loadmat(out_path+os.sep+filename)

    RESULTS_PARAMS = dic["results"]
    mC = RESULTS_PARAMS.shape[1]
    Neff = dic["Neff"]


    propagation_time = 3.0*60*60 #sec 
    rel_time = 3.0*60.0*60 #sec
    dose_integration_time = 10*60 #sec
    dose_sub_step_no = 5
    total_steps = propagation_time / dose_integration_time
    pN = int(rel_time / dose_integration_time)
    print "Running puff model with %d puffs..." % pN
    print "total steps: %d" % total_steps

    relH = 50
    
    nDB = ndb.NuclideDB()
    PUFF_PARAMS = {}
    PUFF_PARAMS["relH"] = 50 #meters
    PUFF_PARAMS["nucl_name"] = "Ar-41"
    PUFF_PARAMS["dose_integration_time"] = dose_integration_time #je to vlastne time_step length
    PUFF_PARAMS["dose_sub_step_no"] = dose_sub_step_no
    PARAMS = {}
    PARAMS["dX"] = 41 #liche, aby sit byla symetricka
    PARAMS["dY"] = 41
    PARAMS["dS"] = 1.0
    PARAMS["puff_number"]     = pN
    #PARAMS["activities"]      = activities
    PARAMS["total_steps"]     = total_steps
    PARAMS["nuclideDB"]       = ndb
    PARAMS["total_steps"] = total_steps
    PARAMS["PUFF_PARAMS"] = PUFF_PARAMS

    
    n = 5

    doseA     = nDB.getDoseA(PARAMS["PUFF_PARAMS"]["nucl_name"])
    doseB     = nDB.getDoseB(PARAMS["PUFF_PARAMS"]["nucl_name"])
    doseMJU   = nDB.getDoseMJU(PARAMS["PUFF_PARAMS"]["nucl_name"])
    doseMJUaE = nDB.getDoseMJUaE(PARAMS["PUFF_PARAMS"]["nucl_name"])
    doseCb    = nDB.getDoseCb(PARAMS["PUFF_PARAMS"]["nucl_name"])

    pN = PARAMS["puff_number"]
    # pro Cython potrebujeme objekt, implementuje buffer protokol (numpy.ndarray):
    models = numpy.zeros((mC, pN), dtype=Puff)

    Q0 = 0.
    a0 = 1. 
    b0 = 0. 
    sc0 = 4  
    wb0 = 0.

    for i in range(mC):
        for j in range(pN):
            H = PARAMS["PUFF_PARAMS"]["relH"]
            vG = nDB.getDepoVG(PARAMS["PUFF_PARAMS"]["nucl_name"])
            lambda_decay = nDB.getLambda(PARAMS["PUFF_PARAMS"]["nucl_name"])
            puff = Puff(id=j,
                Q=Q0, wind_a=a0, wind_b=b0, Pcategory=sc0, wind_c=wb0, # odhadovane parametry
                x0=0., y0=0., H=H, tot_d=1., tot_t=1., dd_total=1., rd_total=1., sigmaxy=0.,
                sigmaz=0., lambda_decay=lambda_decay, vG=vG)
            models[i,j] = puff

################################RECEPTORS
    receptor_path = ".."+os.sep+"data"+os.sep+"receptory_POI"
    receptors_POI = []
    receptors = []

    X, Y, names = loadReceptors(receptor_path)
    number_of_POI_receptors = len(X)
    number_of_VIZU_receptors = PARAMS["dX"] * PARAMS["dY"]

    print "Preparing POI receptors"
    for i in range(len(X)):
        receptors_POI.append((X[i], Y[i], 0.0))

    sourceX =  (PARAMS["dX"]-1)*PARAMS["dS"] / 2.0 * 1000
    sourceY =  (PARAMS["dY"]-1)*PARAMS["dS"] / 2.0 * 1000

    print "Preparing grid receptors"
    for k in range(PARAMS["dX"]):
        for l in range(PARAMS["dY"]):
                xxx = sourceX - (k*1000*PARAMS["dS"])
                yyy = (l*1000*PARAMS["dS"])-sourceY
                receptors.append((xxx, yyy, 0))
   
    receptors = numpy.array(receptors)
#######################################################################  




    METEO = scipy.io.loadmat(".."+os.sep+"data"+os.sep+"meteo")["METEO"]
    METEO_OBS_DICT = scipy.io.loadmat(".."+os.sep+"data"+os.sep+"meteo_obs")
   
    METEO_OBS_WD_REAL = METEO_OBS_DICT["wd_real"][:,0] 
    METEO_OBS_WS_REAL = METEO_OBS_DICT["ws_real"][:,0] 
    METEO_OBS_WD = METEO_OBS_DICT["wd"][:,0]  
    METEO_OBS_WS = METEO_OBS_DICT["ws"][:,0] 
    
    obs_meteo_flag = False #we use meteoforecast for forcing model
    
    gamm_a = dic["gamm_a"] 
    sig_b =  dic["sig_b"]
  
########################################################################    
    start = time.time()
    
    total_DOSE = numpy.zeros((int(propagation_time/dose_integration_time), len(receptors)))
    this_step_DOSE = numpy.zeros((int(propagation_time/dose_integration_time), len(receptors)))

    
    assim_horizon = 18 #number of time steps for assimilation, since then we forecast
    

    for t in range(int(propagation_time/dose_integration_time)): #cyklus pres casove kroky
        total_time_global = t * dose_integration_time
        print "TIME STEP %d, time=%3.2fmin" % (t, total_time_global/60.0)

        if (t < assim_horizon ):
            VAHY = RESULTS_PARAMS[t, :, 0]


                                                         
        for j in range(mC): #loop over particles
            if (t < assim_horizon):        
                for i in range(len(models[j])):
                    models[j][i].wind_a = RESULTS_PARAMS[t, j, 2] #a
                    models[j][i].wind_b = RESULTS_PARAMS[t, j, 3] #b

                if (t < pN):
                    models[j][t].Q = RESULTS_PARAMS[t,j,1] #Q
            else:  #za asimilacnim horizontem uz se vyviji jen podle prechodoveho modelu - predikce                             
                a_old = models[j][0].wind_a
                b_old = models[j][0].wind_b
                a_new = numpy.random.gamma(gamm_a**(-2.0), gamm_a**(2.0) * a_old)
                b_new = numpy.random.normal(b_old, sig_b) 
                      
                for i in range(len(models[j])):                    
                    models[j][i].wind_a = a_new #a
                    models[j][i].wind_b = b_new #b

            

        rec = receptors

        (models, ci_of_models, dose_of_models) = ppe.parallel_puff(j, total_time_global, models, rec, doseA, doseB, doseMJU, doseMJUaE, doseCb, dose_integration_time, dose_sub_step_no, n, METEO, obs_meteo_flag, METEO_OBS_WS, METEO_OBS_WD)

        

        print "waiting to finish all jobs"
        print "FINISHED!"

        this_step_DOSE[t,:] = numpy.sum(numpy.sum(dose_of_models[:,:,:], axis=2), axis=0) / mC
   
    dic = {}
    dic["this_step_DOSE"] = this_step_DOSE
    dic["number_of_POI_receptors"] = number_of_POI_receptors
    dic["number_of_VIZU_receptors"] = number_of_VIZU_receptors


    scipy.io.savemat("%s%s_grid_%d" % (out_path+os.sep,filename, assim_horizon), dic, do_compression=True)    
    print "Saved file %s%s_grid_%d" % (out_path+os.sep,filename, assim_horizon)

if __name__ == "__main__":
    print "Hello World:)"
    out_path = "output"
    
    #comment one of these to filenames to obtain postprocessed results - 2D distribution of doses on terrain 
    
    #filename = "results_AMIS_0"
    filename = "results_NR_0"

    main(out_path, filename)

    
    