#!/usr/bin/env python

from Cython.Distutils import build_ext, Extension
from distutils.core import setup, Command
from glob import glob
import numpy
import os
from os.path import splitext, exists

prefix_dep = "puff_cython"+os.sep
prefix_src = "puff_python"+os.sep
sources = ["puff_tools.py", "ppe.py", "propagate_puff.py"] 
dependencies = [prefix_dep+"puff_tools.pxd", 
                prefix_dep+"ppe.pxd", 
                prefix_dep+"propagate_puff.pxd"] 

ext_options = {
    "cython_directives": {"profile": True, "binding": False},  # add profiling informaiton to generated code
    "cython_c_in_temp": True,
    "extra_compile_args": ["-O2", "-march=core2", "-fopenmp"],
    "extra_link_args": ["-Wl", "-fopenmp"],
    "include_dirs": [numpy.get_include()],
}


ext_modules = [Extension("puff_cython."+splitext(file)[0], [prefix_src+file] + dependencies, **ext_options) for file in sources]

setup(name="puff_cython", packages="puff_cython", ext_modules=ext_modules, cmdclass={"build_ext": build_ext})
      
